/*
    server.cpp  -  Floodcast server socket

    Copyright (c) 2002 by Ryan Cumming
    Copyright (c) 2004 by Stefan Gehn <metz AT gehn.net>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include "server.h"
#include "floodcastconfig.h"
#include "mp3stream.h"

#include <qsocket.h>
#include <qregexp.h>
#include <qserversocket.h>
#include <qtextstream.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kdebug.h>

IcecastServerSocket::IcecastServerSocket(int port, QObject *parent,
	const char *name) : QServerSocket(port, 1, parent, name)
{
	if (!ok())
	{
		// Someone's already bound to our port, complain
		KMessageBox::sorry(0,
			i18n("Floodcast was unable to listen on port %1. Please make sure no" \
				" other applications are using this port and try again.")
					.arg(QString::number(port)));
	}
}

void IcecastServerSocket::newConnection(int socket)
{
	// Let our parent know that we have a new connection
	emit(connection(socket));
}

// ---------------------------------------------------------------------------

IcecastServer::IcecastServer(QObject *parent, const char *name)
	: QObject(parent, name),
	mServerSocket(0),
	mStream(0),
	mActive(false)
{
	kdDebug(66666) << k_funcinfo << endl;

	_sockets.setAutoDelete(true);
	_max_users = FloodcastConfig::serverMaxUsers();

	// This makes a new IcecastServerSocket
	setListenPort(FloodcastConfig::serverPort());
}

IcecastServer::~IcecastServer()
{
	kdDebug(66666) << k_funcinfo << endl;
}

void IcecastServer::addSocket(int socket)
{
	// We have a new connection!

	// Create the socket
	QSocket* s = new QSocket(this);
	s->setSocket(socket);

	// Read client handles HTTP negotiation, they need to do this before they get an audio stream
	connect(s, SIGNAL(readyRead()), this, SLOT(readClient()));
	// We want to know if this connection is closed, to we can reap it
	connect(s, SIGNAL(connectionClosed()), this, SLOT(connectionClosed()));
}

void IcecastServer::readClient()
{
	QSocket* socket = (QSocket*)sender();
	if (socket->canReadLine())
	{
		// Parse the HTTP request
		QStringList tokens = QStringList::split( QRegExp("[\\s]+"), socket->readLine() );
		// Make sure it's a GET
		if (tokens[0] == "GET")
		{
			QTextStream os(socket);
			// HTTP is all Latin1
			os.setEncoding(QTextStream::Latin1);

			if (!mActive)
			{
				// streaming disabled, don't accept incoming connections
				os << "ICY 401 Service Unavailable\r\n";
				socket->flush();
				removeSocket(socket);
			}
			else if (_sockets.count() >= _max_users)
			{
				// Server maxed out
				os << "ICY 503 Server Full\r\n";
				socket->flush();
				removeSocket(socket);
			}
			else
			{
				// We understood the request, and we're an icecast server
				os << "ICY 200 OK\r\n";
				os << "icy-notice1: Noatun Floodcast plugin\r\n";
				os << "icy-notice2: Don't drink and drive\r\n";

				// Dump some stupid config settings
				QString title = FloodcastConfig::streamTitle();
				if (!title.isEmpty())
					os << "icy-name:" << title << "\r\n";

				QString genre = FloodcastConfig::streamGenre();
				if (!genre.isEmpty())
					os << "icy-genre:" << genre << "\r\n";

				QString url = FloodcastConfig::streamURL();
				if (!url.isEmpty())
					os << "icy-url:" << url << "\r\n";

				os << "icy-br:" << QString::number(FloodcastConfig::bitrate()) << "\r\n";

				// We don't do public streams
				// You should probably set up a full Icecast server for
				// anything more than a simple private audio stream
				os << "icy-pub:0\r\n";

				// And this is an MP3 audio stream
				os << "Content-Type: audio/mpeg\r\n\r\n";

				// Will this be our first listener? Fire up the decoder
				if (_sockets.isEmpty())
				{
					if (mStream)
						kdWarning(66666) << k_funcinfo <<
							"Creating a new mp3stream although we already have one!" << endl;

					// Create the MP3 stream
					mStream = new MP3Stream(this, "mp3stream");
					// This signal is called when the encoder has produced new data
					connect(mStream, SIGNAL(newData(char *, int)),
						this, SLOT(transmitData(char *, int)));
					connect(mStream, SIGNAL(encoderDied()), this, SLOT(slotEncoderDied()));
				}

				// Add this socket to the list
				_sockets.append(socket);
				// We don't care about any more read events, once the connection
				// is set up
				disconnect(socket, SIGNAL(readyRead()), this, SLOT(readClient()));

				// Let people that make be monitoring us (ie, the status window)
				// know about this connection
				emit(gainedConnection(socket));
			}
		}
		else
		{
			// Malformed request, dump the socket
			removeSocket(socket);
		}
	}
}

void IcecastServer::connectionClosed()
{
	// The socket died, clean up after it
	removeSocket((QSocket*)sender());
}

void IcecastServer::removeSocket(QSocket *socket)
{
	kdDebug(66666) << k_funcinfo << "removing socket" << endl;
	// Could we remove this from the socket list?
	if (!_sockets.remove(socket))
	{
		kdDebug(66666) << k_funcinfo << "Failed removing socket, just deleting object" << endl;
		// Nope, just delete it and hope for the best
		delete socket;

		// Note: gainedConnection() is only sent when
		// HTTP setup is complete, and a socket is
		// on the sockets list. Obviously, this socket
		// didn't make it that far, so don't emit
		// lostConnection()
	}
	else
	{
		// Yes, was this our last socket?
		if (_sockets.isEmpty())
		{
			kdDebug(66666) << k_funcinfo << "last socket, closing down encoder" << endl;
			// Kill the encoder and aRts object
			delete mStream;
			mStream = 0;
		}

		// Let our monitors know about this connection's tragic death
		emit(lostConnection(socket));
	}
}

void IcecastServer::transmitData(char *buffer, int buflen)
{
	// We have new data to send
	for (QSocket *s = _sockets.first(); s; s = _sockets.next())
	{
		// Is this socket still connected?
		if (s->state() == QSocket::Connected)
		{
			// Yes, send the data off
			if (s->writeBlock(buffer, buflen) == -1)
			{
				removeSocket(s);
			}
		}
		else
		{
			// No, clean up after it
			removeSocket(s);
		}
	}
}

void IcecastServer::restartEncoder()
{
	kdDebug(66666) << k_funcinfo << endl;
	// Restart our encoder object
	if (mStream)
	{
		delete mStream;
		mStream = new MP3Stream(this, "stream");
		connect(mStream, SIGNAL(newData(char *, int)),
			this, SLOT(transmitData(char *, int)));
		connect(mStream, SIGNAL(encoderDied()), this, SLOT(slotEncoderDied()));
	}
}

void IcecastServer::setListenPort(unsigned int port)
{
	if (mServerSocket)
	{
		// Don't recreate the socket if the listening port number didn't change
		if (mServerSocket->ok() && mServerSocket->port() == port)
			return;
	}

	// Destroy our old server socket, if any
	delete mServerSocket;
	// Make a new one
	mServerSocket = new IcecastServerSocket(port, this, "IcecastServersocket");
	// This is called once a new connection is made
	connect(mServerSocket, SIGNAL(connection(int)), this, SLOT(addSocket(int)));
}

int IcecastServer::maxUsers()
{
	return _max_users;
}

void IcecastServer::setMaxUsers(int max)
{
	_max_users = max;
}

void IcecastServer::stop()
{
	kdDebug(66666) << k_funcinfo << endl;
	if (!_sockets.isEmpty())
	{
		if (KMessageBox::questionYesNo(0,
			i18n("Do you really want to disconnect all listeners?"),
			i18n("Floodcast")) == KMessageBox::No)
		{
			emit(statusChange(true));
			return;
		}
	}

	// We don't take new connections, sorry
	mActive = false;
	// And now kick anybody connected :)
	for (QSocket *s = _sockets.first(); s; s = _sockets.next())
	{
		kdDebug(66666) << k_funcinfo << "calling removeSocket()" << endl;
		removeSocket(s);
	}
	emit(statusChange(false));
}

void IcecastServer::start()
{
	kdDebug(66666) << k_funcinfo << endl;
	mActive = true;
	emit(statusChange(true));
}

void IcecastServer::slotEncoderDied()
{
	kdDebug(66666) << k_funcinfo << endl;

	// Duplicated code from stop() to avoid popping up several messageboxes at
	// once (one for the encoder having died and one for the disconnect question)
	mActive = false;
	for (QSocket *s = _sockets.first(); s; s = _sockets.next())
		removeSocket(s);
	emit(statusChange(false));
}

#include "server.moc"
