/*
    status.cpp  -  Floodcast standalone-server status-window

    Copyright (c) 2002 by Ryan Cumming
    Copyright (c) 2004 by Stefan Gehn <metz AT gehn.net>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include <qvgroupbox.h>
#include <qsocket.h>
#include <qtimer.h>
#include <qwhatsthis.h>

#include <kdialogbase.h>
#include <klocale.h>
#include <klistview.h>
#include <kpopupmenu.h>
#include <kdebug.h>

#include "server.h"
#include "status.h"
#include "plugin.h"


HostItem::HostItem(KListView *parent, const QHostAddress &a)
	: KListViewItem(parent)
{
	mAddress = a;
	kdDebug(66666) << k_funcinfo << "HostItem for " << a.toString() << endl;


	mLookup = new QDns(mAddress);
	connect(mLookup, SIGNAL(resultsReady()), this, SLOT(slotLookupComplete()));

	mConnectTime = QDateTime::currentDateTime();

	mUpdateTimer = new QTimer(this, "mUpdateTimer");
	connect(mUpdateTimer, SIGNAL(timeout()), this, SLOT(updateTime()));
	mUpdateTimer->start(10 * 1000);

	setText(0, a.toString());
	updateTime();
}


HostItem::~HostItem()
{
	delete mLookup;
}


void HostItem::slotLookupComplete()
{
	QString name = mAddress.toString();

	QStringList hostNames = mLookup->hostNames();
	QStringList::Iterator it = hostNames.begin();
	while(it != hostNames.end())
	{
		if (!(*it).isEmpty())
		{
			name = (*it);
			break;
		}
		++it;
	}

	setText(0, name);
	kdDebug(66666) << k_funcinfo <<
		"reverse lookup for HostItem IP complete: " << name << endl;
}


void HostItem::updateTime()
{
	unsigned int hours, minutes, seconds;
	QString time_string;

	seconds = mConnectTime.secsTo(QDateTime::currentDateTime());

	minutes = (seconds % 3600) / 60;
	hours = seconds / 3600;

	seconds -= hours * 3600;
	seconds -= minutes * 60;

	/*
	time_string = QString::number(hours) + ":";

	if (minutes < 10)
		time_string += "0";

	time_string += QString::number(minutes);
	*/

	time_string.sprintf(i18n("hours:minutes:seconds", "%d:%.2d:%.2d"), hours, minutes, seconds);

	setText(1, time_string);
}


// ----------------------------------------------------------------------------


FloodcastStatus::FloodcastStatus()
	: KDialogBase(0, "FloodcastStatus", false, i18n("Floodcast Status"), Close)
{
	mServer = 0;

	connect(this, SIGNAL(closeClicked()), SLOT(hide()));

	QVGroupBox *groupBox = new QVGroupBox(i18n("Connected Users"), this);
	groupBox->setMinimumWidth(380);
	setMainWidget(groupBox);

	mListView = new KListView(groupBox);
	mListView->addColumn(i18n("Hostname"));
	mListView->addColumn(i18n("Duration"));
	mListView->setSelectionMode(QListView::Extended);

	QWhatsThis::add(mListView,
		i18n("A list of currently connected users, and how long they have been connected "
			"in minutes. You can forcibly disconnect a user by right clicking on their "
			"list entry selecting \"Disconnect\"."));

	connect(mListView, SIGNAL(contextMenuRequested(QListViewItem *, const QPoint &, int)),
		this, SLOT(showContextMenu(QListViewItem *, const QPoint &)));


	mContextMenu = new KPopupMenu(this, "mContextMenu");
	mContextMenu->insertItem(i18n("&Disconnect"), this, SLOT(disconnectClient()));
}


void FloodcastStatus::setServer(IcecastServer *server)
{
	if (server)
	{
		if (mServer)
			slotServerDestroyed();

		mServer = server;
		connect(mServer, SIGNAL(gainedConnection(QSocket *)),
			this, SLOT(gainedConnection(QSocket *)));
		connect(mServer, SIGNAL(lostConnection(QSocket *)),
			this, SLOT(lostConnection(QSocket *)));
		connect(mServer, SIGNAL(destroyed()), this, SLOT(slotServerDestroyed()));
	}
	else
	{
		slotServerDestroyed();
	}
}


void FloodcastStatus::slotServerDestroyed()
{
	hide();
	disconnect(mServer, 0);
	mServer = 0;
}



void FloodcastStatus::gainedConnection(QSocket *s)
{
	// Make a listview item
	HostItem *list_item = new HostItem(mListView, s->peerAddress());
	mEntryMap[s] = list_item;
}


void FloodcastStatus::lostConnection(QSocket *s)
{
	if (mEntryMap.contains(s))
	{
		delete mEntryMap[s];
		mEntryMap.remove(s);
	}
}


void FloodcastStatus::disconnectClient()
{
	if (!mServer)
		return;

	QMap<QSocket *, HostItem*>::iterator listview_it;
	QValueList<QSocket*>::iterator sockets_it;
	QValueList<QSocket*> dead_sockets;

	for (listview_it = mEntryMap.begin(); listview_it != mEntryMap.end(); ++listview_it)
	{
		if (listview_it.data()->isSelected())
		{
			dead_sockets.push_back(listview_it.key());
		}
	}

	for (sockets_it = dead_sockets.begin();sockets_it != dead_sockets.end(); ++sockets_it)
	{
		mServer->removeSocket(*sockets_it);
	}
}


void FloodcastStatus::showContextMenu(QListViewItem *, const QPoint &pos)
{
	mContextMenu->exec(pos);
}

#include "status.moc"
